### beta.biv represents the beta bivariada density 
##  (a,b,c) evaluated at (x,y)
## it calls function Beta(a,b,c)

### Beta(a,b,c) = [gamma(a)*gamma(b)*gamma(c)]/gamma(a+b+c)

beta.func <- function(a,b,c){
  res <- lgamma(a) + lgamma(b) + lgamma(c) - lgamma(a+b+c)
  return(res)
}

beta.biv <- function(param,x,y){
  ## bivariate beta in log scale
  a <- b <- param[1]
  c <- param[3]
  
  beta <- beta.func(a,b,c)
  
  num <- (a-1)*log(x)+(b-1)*log(y) + (b+c-1)*log(1-x) + (a+c-1)*log(1-y)
  den <- beta + (a+b+c)*log(1-x*y)
  value <- num-den
  
  return(value)
}

beta.biv.i <- function(par,param,x1=NULL,y1=NULL,log=F,tipo=1){
  ## bivariate beta in log scale
  ## tipo = 1 entao bivariada
  ## tipo = 2 entao integro em x
  ## tipo = 3 entao integro em y
  
  if (tipo==1){
    x <- par[1] 
    y <- par[2]
  }
  
  if (tipo==2)
  {
    x <- par
    y <- rep(y1,length(x))
  }
  
  if (tipo==3)
  {
    y <- par
    x <- rep(x1,length(y))
  }
  
  # print(par)
  # print(cat("x= ",x,"y= ",y,"x1= ", x1,"y1=  ",y1))
  
  a <- b <- param[1]
  c <- param[3]
  
  beta <- beta.func(a,b,c)
  
  num <- (a-1)*log(x)+(b-1)*log(y) + (b+c-1)*log(1-x) + (a+c-1)*log(1-y)
  den <- beta + (a+b+c)*log(1-x*y)
  value <- num-den
  if (log==F) 
    value <- exp(value)
  return(value)
}

beta.biv.mcmc <- function(x=NULL,y=NULL,a,b,c,log=F,tipo=1){
  ## bivariate beta in log scale
  ## tipo = 1 entao bivariada
  ## tipo = 2 entao integro em x
  ## tipo = 3 entao integro em y
  
  beta <- beta.func(a,b,c)
  
  num <- (a-1)*log(x)+(b-1)*log(y) + (b+c-1)*log(1-x) + (a+c-1)*log(1-y)
  den <- beta + (a+b+c)*log(1-x*y)
  value <- num-den
  if (log==F) 
    value <- exp(value)
  return(value)
}


f00 <- function(param,log=F,upper=0.01){
  value <- adaptIntegrate(beta.biv.i, lowerLimit= rep(0.0001,2), upperLimit = rep(upper,2), 
                          param,x1=NULL,y1=NULL, log=log,tipo=1, fDim = 1, maxEval = 100000, absError=0, 
                          doChecking=FALSE)$integra
  #  print(betabiv)	
  return(value)   
}

beta.biv.i.mcmc <- function(x,y,param,log){
  
  
  a <- b <- param[1]
  c <- param[3]
  
  beta <- beta.func(a,b,c)
  
  num <- (a-1)*log(x)+(b-1)*log(y) + (b+c-1)*log(1-x) + (a+c-1)*log(1-y)
  den <- beta + (a+b+c)*log(1-x*y)
  value <- num-den
  if (log==F) 
    value <- exp(value)
  return(value)
}

f00.mcmc <- function(param,log=F,upper=0.01,N,pontos){
  value <- (upper^2)*sum(outer(pontos,pontos,beta.biv.mcmc,param[1],param[1],param[3],log=F))/N^2
  return(value)   
}

f0b.i <- function(param,y,log=F,upper){
  #print(cat(param,"  ",y))
  a <- param[1]
  c <- param[3]
  
  value <- NULL
  for (i in 1:length(y)){
    value[i] <- integrate(beta.biv.i,0.0001,upper,param=c(a,a,c),x1=NULL,y1=y[i],log=log,tipo=2)$value
  } 
  #if (log==F)
  #   value <- exp(value)
  return(value)   
}

f0b.mcmc <- function(param,y,log=F,upper,N,pontos){
  a <- b <- param[1]
  c <- param[3]
  
  #value <- rep(0,length(y))
  #for (i in 1:length(y)){
  # value[i] <- upper*sum(apply(matrix(pontos),1,
  #                           function(v){beta.biv.mcmc(x=v,y=y[i],a,b,c,log=log,tipo=2)}))/N
  #}
  #return(value)   
  
  return(upper*apply(outer(pontos,y,beta.biv.mcmc,param[1],param[1],param[3],log=F,tipo=2),2,sum)/N)
}

fb0.i <- function(param,x,log=F,upper){
  a <- param[1]
  c <- param[3]
  
  value <- rep(0,length(y))
  for (i in 1:length(x)){
    value[i] <- integrate(beta.biv.i,0.0001,upper,param=c(a,a,c),x1=x[i],y1=NULL,log=log,tipo=3)$value
  } 
  #if (log==F)
  #   value <- exp(value)
  return(value)   
}

fb0.mcmc <- function(param,x,log=F,upper,N,pontos){
  a <- b <- param[1]
  c <- param[3]
  
  #value <- rep(0,length(y))
  #for (i in 1:length(x)){
  #     value[i] <- upper*sum(apply(matrix(pontos),1,function(v){beta.biv.mcmc(x=x[i],y=v,a,b,c,log=log,tipo=3)}))/N
  #  }
  #  
  #return(value)   
  
  return(upper*apply(outer(pontos,x,beta.biv.mcmc,param[1],param[1],param[3],log=F,tipo=3),2,sum)/N)
}

library(cubature)


count <- function(data,condition){
  tt <- eval(parse(text=condition))
  sum <- sum(tt)
  is <- which(tt==1)       
  d <- data[,is]
  return(list("sum"=sum,"data"=d))}


listcount <- function(data,string){   
  list <- apply(data,3,count,string)     
  nj <- unlist(lapply(list,function(l){return(l$sum)}))
  ions <- which(nj != 0)
  if (sum(ifelse(nj==0,1,0)) >= 1)
    nj <- nj[-which(nj==0)] # remove zeros counts from nj*.*
  return(list(list=list,nj=nj,ions=ions))
}


targfunc.marg <- function(param,ppi.t1,ppi.t2,res1.00, res2.00, res1.0b, res2.0b, res1.b0,
                          res2.b0, res1.bb, res2.bb, L,nions,cc,N,pontos){
  
  #print(cat("max z.1 = ",max(z.1), "max z.2 = ",max(z.2)))
  
  list1.00 <- res1.00$list
  nj1.00 <- res1.00$nj
  ions1.00 <- res1.00$ions
  
  list2.00 <- res2.00$list
  nj2.00 <- res2.00$nj
  ions2.00 <- res2.00$ions
  
  # consider y's with both reps in (0,1)
  list1.bb <- res1.bb$list
  nj1.bb <- res1.bb$nj
  ions1.bb <- res1.bb$ions
  
  list2.bb <- res2.bb$list
  nj2.bb <- res2.bb$nj
  ions2.bb <- res2.bb$ions
  
  list1.0b <- res1.0b$list
  nj1.0b <- res1.0b$nj
  ions1.0b <- res1.0b$ions
  
  list1.b0 <- res1.b0$list
  nj1.b0 <- res1.b0$nj
  ions1.b0 <- res1.b0$ions
  
  list2.0b <- res2.0b$list
  nj2.0b <- res2.0b$nj
  ions2.0b <- res2.0b$ions
  
  list2.b0 <- res2.b0$list
  nj2.b0 <- res2.b0$nj
  ions2.b0 <- res2.b0$ions
  
  like.inc2 <- like.inc1 <-0
  
  for (j in 1:n.ions){
    soma.k1 <- 0
    partlike1.0b <- partlike1.b0 <- partlike1.00 <- partlike1.bb <- 1
    soma.k2 <- 0
    partlike2.0b <- partlike2.b0 <- partlike2.00 <- partlike2.bb <- 1
    
    for (l in 1:L){
      
      a <- b <- param[2*l-1]
      c <- param[2*l]
      f00.i <- f00.mcmc(c(a,b,c),log=F,upper=0.01,N,pontos)
      
      if (j %in% ions1.00){
        partlike1.00 <- rep(f00.i,nj1.00[ions1.00==j])}
      
      if (j %in% ions2.00)
      {
        partlike2.00 <- rep(f00.i,nj2.00[ions2.00==j])}
      
      
      if (j %in% ions1.bb)
      {
        dds <- as.matrix(list1.bb[[j]]$data)
        partlike1.bb <- (0.01^2)*exp(beta.biv(c(a,b,c),dds[1,]+cc,dds[2,]+cc))}
      
      
      if (j %in% ions2.bb){
        dds <- as.matrix(list2.bb[[j]]$data)
        partlike2.bb <- (0.01^2)*exp(beta.biv(c(a,b,c),dds[1,]+cc,dds[2,]+cc))}
      
      if (j %in% ions1.0b){
        dds <- as.matrix(list1.0b[[j]]$data)
        partlike1.0b <- (0.01)*f0b.mcmc(c(a,b,c),dds[dds>0]+cc,log=F,upper=0.01,N,pontos)}
      
      
      if (j %in% ions2.0b){
        dds <- as.matrix(list2.0b[[j]]$data)
        partlike2.0b <- (0.01)*f0b.mcmc(c(a,b,c),dds[dds>0]+cc,log=F,upper=0.01,N,pontos)}
      
      if (j %in% ions1.b0){
        dds <- as.matrix(list1.b0[[j]]$data)
        partlike1.b0 <- 0.01*fb0.mcmc(c(a,b,c),dds[dds>0]+cc,log=F,upper=0.01,N,pontos)} 
      
      if (j %in% ions2.b0){
        dds <- as.matrix(list2.b0[[j]]$data)
        partlike2.b0 <- 0.01*fb0.mcmc(c(a,b,c),dds[dds>0]+cc,log=F,upper=0.01,N,pontos)} 
      
      soma.k2 <- soma.k2 + (ppi.t1[l]*(prod(partlike2.00)*prod(partlike2.0b)*
                                         prod(partlike2.b0)*prod(partlike2.bb)))
      soma.k1 <- soma.k1 + (ppi.t2[l]*(prod(partlike1.00)*prod(partlike1.0b)*
                                         prod(partlike1.b0)*prod(partlike1.bb)))
      
      #}
    } ##closes l
    
    like.inc2 <- like.inc2 + log(soma.k2)
    like.inc1 <- like.inc1 + log(soma.k1)
    print(cat(j,"  ",log(soma.k1),log(soma.k2))) 
    
  } ### closes j
  like.inc <- like.inc1+like.inc2
  return(like.inc)
}


###################### targfunc.biv.em.unica is the 
##################### log-likelihood function 
###################  EM algorithm

targfunc <- function(a,b,c,res1.00, res2.00, res1.0b, 
                     res2.0b, res1.b0, res2.b0, res1.bb, res2.bb, z.1, 
                     z.2, l ,nions,cc,pontos,N){
  part00.1 <- part00.2 <- part0b.1 <- part0b.2 <- partb0.1 <- partb0.2 <- partbb.1 <- partbb.2 <- rep(0,nions)
  
  f00.i <- f00.mcmc(c(a,b,c),log=F,upper=0.01,N,pontos)
  
  part00.1[ions1.00] <- apply(as.matrix(res1.00$ions),1,function(x){ z.1[x,l]*res1.00$list[[x]]$sum*log(f00.i)})
  part00.2[ions2.00] <- apply(as.matrix(res2.00$ions),1,function(x){ z.2[x,l]*res2.00$list[[x]]$sum*log(f00.i)})
  
  partbb.1[ions1.bb] <- apply(as.matrix(res1.bb$ions),1,
                              function(x){ dds <- as.matrix(res1.bb$list[[x]]$data);
                              z.1[x,l]*sum(log(0.01^2)+beta.biv(c(a,b,c),dds[1,]+cc,dds[2,]+cc))})
  
  partbb.2[ions2.bb] <- apply(as.matrix(res2.bb$ions),1,
                              function(x){ dds <- as.matrix(res2.bb$list[[x]]$data);
                              z.2[x,l]*sum(log(0.01^2)+beta.biv(c(a,b,c),dds[1,]+cc,dds[2,]+cc))})
  
  part0b.1[ions1.0b] <- apply(as.matrix(res1.0b$ions),1,
                              function(x){dds <- as.matrix(res1.0b$list[[x]]$data);
                              z.1[x,l]*sum(log(0.01)+log(f0b.mcmc(c(a,b,c),dds[dds>0]+cc,log=F,upper=0.01,N=100,pontos)))})
  
  part0b.2[ions2.0b] <- apply(as.matrix(res2.0b$ions),1,
                              function(x){ dds <- as.matrix(res2.0b$list[[x]]$data);
                              z.2[x,l]*sum(log(0.01)+log(f0b.mcmc(c(a,b,c),dds[dds>0]+cc,log=F,upper=0.01,N=100,pontos)))})
  
  partb0.1[ions1.b0] <- apply(as.matrix(res1.b0$ions),1,
                              function(x){ dds <- as.matrix(res1.b0$list[[x]]$data);
                              z.1[x,l]*sum(log(0.01)+log(fb0.mcmc(c(a,b,c),dds[dds>0]+cc,log=F,upper=0.01,N=100,pontos)))})
  
  partb0.2[ions2.b0] <- apply(as.matrix(res2.b0$ions),1,
                              function(x){ dds <- as.matrix(res2.b0$list[[x]]$data);
                              z.2[x,l]*sum(log(0.01)+log(fb0.mcmc(c(a,b,c),dds[dds>0]+cc,log=F,upper=0.01,N=100,pontos)))})
  
  res <- -sum(part00.1+part00.2+part0b.1+part0b.2+partb0.1+partb0.2+partbb.1+partbb.2)
  return(res)
}

targfunc.biv.em.unica <- function(param,res1.00, res2.00, res1.0b, 
                                  res2.0b, res1.b0, res2.b0, res1.bb, res2.bb, z.1, 
                                  z.2, L ,nions,cc,pontos,N){
  
  res <- 0
  for (l in 1:L){
    part00.1 <- part00.2 <- part0b.1 <- part0b.2 <- partb0.1 <- partb0.2 <- partbb.1 <- partbb.2 <- rep(0,nions)
    
    a <- b <- param[2*l-1]
    c <- param[2*l]
    
    f00.i <- f00.mcmc(c(a,b,c),log=F,upper=0.01,N,pontos)
    
    part00.1[res1.00$ions] <- apply(as.matrix(res1.00$ions),1,function(x){ z.1[x,l]*res1.00$list[[x]]$sum*log(f00.i)})
    part00.2[res2.00$ions] <- apply(as.matrix(res2.00$ions),1,function(x){ z.2[x,l]*res2.00$list[[x]]$sum*log(f00.i)})
    
    partbb.1[res1.bb$ions] <- apply(as.matrix(res1.bb$ions),1,
                                    function(x){ dds <- as.matrix(res1.bb$list[[x]]$data);
                                    z.1[x,l]*sum(log(0.01^2)+beta.biv(c(a,b,c),dds[1,]+cc,dds[2,]+cc))})
    
    partbb.2[res2.bb$ions] <- apply(as.matrix(res2.bb$ions),1,
                                    function(x){ dds <- as.matrix(res2.bb$list[[x]]$data);
                                    z.2[x,l]*sum(log(0.01^2)+beta.biv(c(a,b,c),dds[1,]+cc,dds[2,]+cc))})
    
    part0b.1[res1.0b$ions] <- apply(as.matrix(res1.0b$ions),1,
                                    function(x){dds <- as.matrix(res1.0b$list[[x]]$data);
                                    z.1[x,l]*sum(log(0.01)+log(f0b.mcmc(c(a,b,c),dds[dds>0]+cc,log=F,upper=0.01,N=N,pontos)))})
    
    part0b.2[res2.0b$ions] <- apply(as.matrix(res2.0b$ions),1,
                                    function(x){ dds <- as.matrix(res2.0b$list[[x]]$data);
                                    z.2[x,l]*sum(log(0.01)+log(f0b.mcmc(c(a,b,c),dds[dds>0]+cc,log=F,upper=0.01,N=100,pontos)))})
    
    partb0.1[res1.b0$ions] <- apply(as.matrix(res1.b0$ions),1,
                                    function(x){ dds <- as.matrix(res1.b0$list[[x]]$data);
                                    z.1[x,l]*sum(log(0.01)+log(fb0.mcmc(c(a,b,c),dds[dds>0]+cc,log=F,upper=0.01,N=100,pontos)))})
    
    partb0.2[res2.b0$ions] <- apply(as.matrix(res2.b0$ions),1,
                                    function(x){ dds <- as.matrix(res2.b0$list[[x]]$data);
                                    z.2[x,l]*sum(log(0.01)+log(fb0.mcmc(c(a,b,c),dds[dds>0]+cc,log=F,upper=0.01,N=100,pontos)))})
    res <- res-sum(part00.1+part00.2+part0b.1+part0b.2+partb0.1+partb0.2+partbb.1+partbb.2)
    
  }
  #return(sum(apply(matrix(seq(1:L)),1,function(l){targfunc(a = param[2*l-1],b= param[2*l-1],c = param[2*l]
  #                                                         ,res1.00, res2.00, res1.0b, 
  #                                                         res2.0b, res1.b0, res2.b0, res1.bb, res2.bb, z.1, 
  #                                                         z.2, l ,nions,cc,pontos,N)})))
  
  return(res)
}


EMopti <- function(data1,data2, L, n.ions,mle.biv.1,ppi.t1,ppi.t2,cc) {
  
  # print(cat("comeca ",mle.biv.1))
  res1.00 <- listcount(data1,'ifelse(data[1,]==0 & data[2,]==0,1,0)')
  list1.00 <- res1.00$list
  nj1.00 <- res1.00$nj
  ions1.00 <- res1.00$ions
  
  res2.00 <- listcount(data2,'ifelse(data[1,]==0 & data[2,]==0,1,0)')
  list2.00 <- res2.00$list
  nj2.00 <- res2.00$nj
  ions2.00 <- res2.00$ions
  
  # consider y's with both reps in (0,1)
  res1.bb <- listcount(data1,'ifelse(data[1,]>0 & data[1,]<1 & data[2,]>0 & data[2,]<1,1,0)')
  list1.bb <- res1.bb$list
  nj1.bb <- res1.bb$nj
  ions1.bb <- res1.bb$ions
  
  res2.bb <- listcount(data2,'ifelse(data[1,]>0 & data[1,]<1 & data[2,]>0 & data[2,]<1,1,0)')
  list2.bb <- res2.bb$list
  nj2.bb <- res2.bb$nj
  ions2.bb <- res2.bb$ions
  
  res1.0b <- listcount(data1,'ifelse((data[2,]>0 & data[2,]<1 & data[1,]==0),1,0)')
  list1.0b <- res1.0b$list
  nj1.0b <- res1.0b$nj
  ions1.0b <- res1.0b$ions
  
  res1.b0 <- listcount(data1,'ifelse((data[1,]>0 & data[1,]<1 & data[2,]==0),1,0)')
  list1.b0 <- res1.b0$list
  nj1.b0 <- res1.b0$nj
  ions1.b0 <- res1.b0$ions
  
  res2.0b <- listcount(data2,'ifelse((data[2,]>0 & data[2,]<1 & data[1,]==0),1,0)')
  list2.0b <- res2.0b$list
  nj2.0b <- res2.0b$nj
  ions2.0b <- res2.0b$ions
  
  res2.b0 <- listcount(data2,'ifelse((data[1,]>0 & data[1,]<1 & data[2,]==0),1,0)')
  list2.b0 <- res2.b0$list
  nj2.b0 <- res2.b0$nj
  ions2.b0 <- res2.b0$ions
  ####################### Initializing parameters #########################
  
  ## z is the indicator vector - latent variable
  
  #	z <- matrix(rep(.5,(length(nj.31)+length(nj.32))*L),nrow=(length(nj.31)+length(nj.32)))
  
  
  z <- matrix(rep(.5,2*n.ions*L),nrow=2*n.ions)
  z.1 <- matrix(rep(.5,n.ions*L),nrow=n.ions)
  z.2 <- matrix(rep(.5,n.ions*L),nrow=n.ions)
  
  lastlike <- lastmle <- 0
  currlike <- 10^6
  count <- 0
  N <- 10^2
  pontos <- seq(0.0001,0.01,length.out=N)
  
  while(abs(lastlike-currlike) > 10e-4){
    lastmle <- c(mle.biv.1)
    lastlike <- currlike
    #print(ppi.t1) 
    ## while(error > 10e-5){
    count <- count + 1
    print(count)
    ## M-step using "nlm" function ##
    
    ## E-step to update the z's ##
    for (l in 1:L){
      
      a <- b <- mle.biv.1[2*l-1]
      c <- mle.biv.1[2*l]
      
      z.1[,l] <- 0
      z.2[,l] <- 0
      f00.i <- f00.mcmc(c(a,b,c),log=F,upper=0.01,N=N,pontos)
      
      z.1[ions1.00,l] <- apply(as.matrix(res1.00$ions),1,function(x){ z.1[x,l]+res1.00$list[[x]]$sum*log(f00.i)})
      z.2[ions2.00,l] <- apply(as.matrix(res2.00$ions),1,function(x){ z.2[x,l]+res2.00$list[[x]]$sum*log(f00.i)})
      z.1[ions1.bb,l] <- apply(as.matrix(res1.bb$ions),1,
                               function(x){ dds <- as.matrix(res1.bb$list[[x]]$data);
                               z.1[x,l]+sum(log(0.01^2)+beta.biv(c(a,b,c),dds[1,]+cc,dds[2,]+cc))})
      
      z.2[ions2.bb,l] <- apply(as.matrix(res2.bb$ions),1,
                               function(x){ dds <- as.matrix(res2.bb$list[[x]]$data);
                               z.2[x,l]+sum(log(0.01^2)+beta.biv(c(a,b,c),dds[1,]+cc,dds[2,]+cc))})
      
      
      z.1[ions1.0b,l] <- apply(as.matrix(res1.0b$ions),1,
                               function(x){dds <- as.matrix(res1.0b$list[[x]]$data);
                               z.1[x,l]+sum(log(0.01)+log(f0b.mcmc(c(a,b,c),dds[dds>0]+cc,log=F,upper=0.01,N=N,pontos)))})
      
      z.2[ions2.0b,l] <- apply(as.matrix(res2.0b$ions),1,
                               function(x){dds <- as.matrix(res2.0b$list[[x]]$data);
                               z.2[x,l] + sum(log(0.01)+log(f0b.mcmc(c(a,b,c),dds[dds>0]+cc,log=F,upper=0.01,N=N,pontos)))})
      
      
      z.1[ions1.b0,l] <- apply(as.matrix(res1.b0$ions),1,
                               function(x){ dds <- as.matrix(res1.b0$list[[x]]$data);
                               z.1[x,l] + sum(log(0.01)+log(fb0.mcmc(c(a,b,c),dds[dds>0]+cc,log=F,upper=0.01,N=N,pontos)))})
      
      z.2[ions2.b0,l] <- apply(as.matrix(res2.b0$ions),1,
                               function(x){ dds <- as.matrix(res2.b0$list[[x]]$data);
                               z.2[x,l] + sum(log(0.01)+log(fb0.mcmc(c(a,b,c),dds[dds>0]+cc,log=F,upper=0.01,N=N,pontos)))})
      
      z.1[,l] <- z.1[,l]+log(ppi.t1[l])
      z.2[,l] <- z.2[,l]+log(ppi.t2[l])
      
      z.1[,l] <- exp(z.1[,l])
      z.2[,l] <- exp(z.2[,l])
      
    } # end for l
    
    z.1 <- z.1/apply(z.1,1,sum,na.rm=T)
    z.2 <- z.2/apply(z.2,1,sum,na.rm=T)
    
    ### calculando p
    for (l in 1:L){
      ppi.t1[l] <- sum(z.1[,l])
      ppi.t2[l] <- sum(z.2[,l])
    }
    ppi.t1 <- ppi.t1/sum(ppi.t1)
    ppi.t2 <- ppi.t2/sum(ppi.t2)
    
    
    #    grad.num <- function(mle.biv.1,res1.00, res2.00, res1.0b, 
    #                         res2.0b, res1.b0, res2.b0, res1.bb, res2.bb, z.1, z.2, L ,nions){
    #    value <- grad(function(param) targfunc.biv.em.unica(param,res1.00, res2.00, res1.0b, 
    #         res2.0b, res1.b0, res2.b0, res1.bb, res2.bb, z.1, z.2, L ,nions,cc), mle.biv.1)
    #      return(value)
    #    }     
    #maxlike.biv.1 <- optim(mle.biv.1, targfunc.biv.em.unica, 
    #                       gr = grad.num, res1.00, res2.00, res1.0b, 
    #                       res2.0b, res1.b0, res2.b0, res1.bb, res2.bb, z.1, 
    #                       z.2, L,n.ions, method = "L-BFGS-B", control=list(fnscale=1),
    #                       lower = rep(0.001,2*L),
    #                       upper=rep(c(50,300),L))
    
    
    #param <- vero <- matrix(data=rep(0,L*10),nrow=2*L)
    #param <- apply(lim,1,function(x){u<-runif(10,0.001,x)})
    
    #vero <- apply(param,1,function(x){targfunc.biv.em.unica(x,res1.00, res2.00, res1.0b, 
    #                                                        res2.0b, res1.b0, res2.b0, res1.bb, res2.bb, z.1, 
    #                                                        z.2, L ,n.ions)})
    
    #print(param)
    #mle.init <- param[which(vero==min(vero)),]
    
    # ptm <- proc.time()         
    
    maxlike.biv.1 <- optim(mle.biv.1, targfunc.biv.em.unica, 
                           gr = NULL, res1.00, res2.00, res1.0b, 
                           res2.0b, res1.b0, res2.b0, res1.bb, res2.bb, z.1, 
                           z.2, L,n.ions,cc,pontos,N, method = "L-BFGS-B", control=list(maxit=50,fnscale=40000),
                           lower = rep(c(0.001,0.001),L),
                           upper=rep(c(100,400),L))
    # proc.time()-ptm      
    mle.biv.1 <- maxlike.biv.1$par ## mle contains alpha's and beta's
    #print(cat("depois ",maxlike.biv.1$par))
    
    print(c(mle.biv.1,ppi.t1,ppi.t2))
    currlike <-  maxlike.biv.1$value
    
    print(currlike)
  } ## close while
  
  print(c("EM has converged after", count, "times"))
  print(c('L=',L))
  
  final.beta <- c(mle.biv.1)
  
  like.marg <- targfunc.marg(param=final.beta,ppi.t1,ppi.t2,res1.00, res2.00, 
                             res1.0b, res2.0b, res1.b0, res2.b0,
                             res1.bb, res2.bb, L,nions,cc,N,pontos)
  
  
  n1 <- sum(res1.00$nj)+sum(res1.0b$nj)+sum(res1.b0$nj)+sum(res1.bb$nj) ## trt 1
  n2 <- sum(res2.00$nj)+sum(res2.0b$nj)+sum(res2.b0$nj)+sum(res2.bb$nj) ## trt2
  n <- n1+n2 ### n=1294
  bic <- -2*like.marg+(4*L-2)*log(n)
  aic <- -2*like.marg+2*(4*L-2)
  
  # en <- - sum(z.1*log(z.1))-sum(z.2*log(z.2))
  
  en <- -sum(z.1[z.1 > 0]*log(z.1[z.1 > 0]))-sum(z.2[z.2 > 0]*log(z.2[z.2 > 0]))
  en2.part1 <- t(apply(matrix(1:n.ions,n.ions,1),1,
                       function(x,z.1,z.2){apply(rbind(z.1[x,],z.2[x,]),2,max)},z.1,z.2))
  en2 <- -sum(log(en2.part1[en2.part1 > 0]))
  
  icl.bic <- bic+2*en
  icl.bic2 <- bic+2*en2
  
  print(n1)
  print(n2)
  print(aic)
  print(bic)
  print(icl.bic)  
  print(icl.bic2)
  
  print(final.beta)
  print(en2)
  return(list(data=final.beta,ppi.t1=ppi.t1,ppi.t2=ppi.t2,maxlike=currlike, 
              like.marg=like.marg,wstar=cbind(z.1,z.2),n1=n1,n2=n2,aic=aic,bic=bic,icl.bic=icl.bic,
              icl.bic2=icl.bic2,en2=en2))
}


#### MAIN PROGRAM ####

nvacas.1 <- 7
nvacas.2 <- 10
nions <- n.ions <- 76

ii <- rep(1:n.ions,each=2)

dif5 <- c(14,16,23,45,50,58,62,63,70,74)
l <- L <- 5
mle.biv.1 <-  rep(c(5,50),5)+rnorm(10)

### reading data

d <- read.table("uterus-cows.csv",header=T)
d2 <- d[,c(2:dim(d)[2])]

## Treatment 1
trat1<-dt1 <- d2[seq(1,nvacas.1*rep,by=1),]
dt1 <- dt1/100
data1 <- array(unlist(dt1),dim=c(rep,nvacas.1,n.ions)) # rep,vaca,ion

## Treatment 2                   
trat2<-dt2 <- d2[seq(nvacas.1*rep+1,(nvacas.1+nvacas.2)*rep,by=1),]
dt2 <- dt2/100
data2 <- array(unlist(dt2),dim=c(rep,10,n.ions)) # rep,vaca,ion


### setting initial values
###   
lastlike <- 0
fit5 <- EMopti(data1,data2, L, n.ions,mle.biv.1,ppi.t1=rep(1/L,L),ppi.t2=rep(1/L,L),cc=0.005)



